#include <stdio.h>

#define PMAX 100027
#define BMAX 63

long long prod[PMAX], minv[PMAX], pw2[BMAX];
long long A, B, P, X, v, p1, p2, p3, v1, v2, v3;
int Q;

inline long long getPow(long long Z)
{
	long long X, rez = 0;

	X = Z;
	rez = 0;
	while (X >= P)
	{
		X /= P;
		rez += X;
	}

	return rez;
}

inline long long raiseToPow(long long U, long long V)
{
	long long rez = 1;
	int bit;

	if (V == 0)
		return 1;

	for (bit = BMAX - 1; bit >= 0; bit--)
	{
		rez = (rez * rez) % P;
		if ((V & pw2[bit]) > 0)
			rez = (rez * U) % P;
	}

	return rez;
}

inline long long getNumber(long long Z)
{
	long long cnt, rez1;

	if (Z <= 1)
		return 1;

	cnt = Z / P;
	rez1 = (raiseToPow(prod[P-1], cnt) * prod[Z % P]) % P;
	return (rez1 * getNumber(Z / P)) % P;
}

long long extendedEuclid(long long a, long long b, long long *x, long long *y)
{
	long long x1, y1, d;

	if (b == 0)
	{
		*x = 1;
		*y = 0;
		return a;
	}
	else
	{
		d = extendedEuclid(b, a % b, &x1, &y1);
		*x = y1;
		*y = x1 - (a/b)*y1;
		return d;
	}
}

long long modularInverse(long long a)
{
	long long x, y, d;

	d = extendedEuclid(P, a, &x, &y);

	while (y<0)
		y = y + P;

	return (y % P);
}

void doPreprocessing(void)
{
	int i;

	prod[0] = prod[1] = 1 % P;
	for (X = 2; X <= P - 1; X++)
		prod[X] = (X * prod[X - 1]) % P;

	pw2[0] = 1;
	for (i = 1; i < BMAX; i++)
		pw2[i] = pw2[i - 1] * 2;

	for (X = 1; X <= P - 1; X++)
		minv[X] = modularInverse(X);
}

int main()
{
	freopen("jap2.in", "r", stdin);
	freopen("jap2.out", "w", stdout);

	scanf("%lld %d", &P, &Q);

	doPreprocessing();

	while (Q--)
	{
		scanf("%lld %lld", &A, &B);
		
		p1 = getPow(A);
		p2 = getPow(B);
		p3 = getPow(A - B);

		//fprintf(stderr, "p1=%lld, p2=%lld, p3=%lld, p2+p3=%lld\n", p1, p2, p3, p2 + p3);

		if (p1 > (p2 + p3))
			printf("0\n");
		else
		{
			v1 = getNumber(A);
			v2 = getNumber(B);
			v3 = getNumber(A - B);

			v1 = (v1 * minv[(v2 * v3) % P]) % P;
			printf("%lld\n", v1);
		}
	}

	return 0;
}
