#include <stdio.h>
#include <vector>

using namespace std;

#define NMAX 100100
#define MMAX 200100

vector<pair<int, int> > vone, vzero;
char c[2 * MMAX], selOne[MMAX], selZero[MMAX];
int e[2 * MMAX], nextNeigh[2 * MMAX];
int firstNeigh[NMAX], lastNeigh[NMAX], cc[NMAX], Q[NMAX];
int tata[NMAX], nnodes[NMAX];
int i, j, p, q, head, tail, w, ncc, N, M, K;

void readInput(void)
{
	freopen("karb.in", "r", stdin);
	scanf("%d %d %d", &N, &M, &K);

	memset(firstNeigh, 0, sizeof(firstNeigh));
	memset(lastNeigh, 0, sizeof(lastNeigh));

	vzero.clear();
	vone.clear();

	q = 0;
	for (p = 1; p <= M; p++)
	{
		scanf("%d %d %d", &i, &j, &w);

		if (w == 0)
			vzero.push_back(make_pair(i, j));
		else
			vone.push_back(make_pair(i, j));

		q++;
		e[q] = j;
		c[q] = w;
		if (!firstNeigh[i])
			firstNeigh[i] = lastNeigh[i] = q;
		else
		{
			nextNeigh[lastNeigh[i]] = q;
			lastNeigh[i] = q;
		}

		q++;
		e[q] = i;
		c[q] = w;
		if (!firstNeigh[j])
			firstNeigh[j] = lastNeigh[j] = q;
		else
		{
			nextNeigh[lastNeigh[j]] = q;
			lastNeigh[j] = q;
		}
	}
}

void computeConnectedComponents(void)
{
	memset(cc, 0, sizeof(cc));

	ncc = 0;

	for (i = 1; i <= N; i++)
		if (!cc[i])
		{
			ncc++;
			cc[i] = ncc;
			Q[head = tail = 1] = i;

			while (head <= tail)
			{
				p = Q[head];
				q = firstNeigh[p];

				while (1)
				{
					if (c[q] == 0 && !cc[e[q]])
					{
						cc[e[q]] = ncc;
						tail++;
						Q[tail] = e[q];
					}

					if (q == lastNeigh[p])
						break;
					else
						q = nextNeigh[q];
				}

				head++;
			}
		}
}

int Union(int x, int y)
{
	int tx, ty, troot;

	// find the tree roots
	tx = x;
	while (tata[tx] > 0)
		tx = tata[tx];

	ty = y;
	while (tata[ty] > 0)
		ty = tata[ty];

	if (tx == ty)
		return 0;

	// union by rank
	if (nnodes[tx] > nnodes[ty])
	{
		tata[ty] = tx;
		nnodes[tx] += nnodes[ty];
		troot = tx;
	}
	else
	{
		tata[tx] = ty;
		nnodes[ty] += nnodes[tx];
		troot = ty;
	}

	// path compression
	while (x != troot)
	{
		tx = tata[x];
		tata[x] = troot;
		x = tx;
	}

	while (y != troot)
	{
		ty = tata[y];
		tata[y] = troot;
		y = ty;
	}

	return 1;
}

void computeInitialOneEdges(void)
{
	memset(selOne, 0, sizeof(selOne));

	// initialize disjoint sets
	for (i = 1; i <= ncc; i++)
	{
		tata[i] = 0;
		nnodes[i] = 1;
	}

	for (p = vone.size() - 1; p >= 0; p--)
		if (Union(cc[vone[p].first], cc[vone[p].second]))
			selOne[p] = 1;
}

void computeAllOneEdges(void)
{
	int cnt = 0;

	// initialize disjoint sets
	for (i = 1; i <= N; i++)
	{
		tata[i] = 0;
		nnodes[i] = 1;
	}

	for (p = vone.size() - 1; p >= 0; p--)
		if (selOne[p])
		{
			Union(vone[p].first, vone[p].second);
			cnt++;
		}

	for (p = vone.size() - 1; p >= 0 && cnt < K; p--)
		if (!selOne[p])
		{
			if (Union(vone[p].first, vone[p].second))
			{
				selOne[p] = 1;
				cnt++;
			}
		}
}

void computeZeroEdges(void)
{
	memset(selZero, 0, sizeof(selZero));

	for (p = vzero.size() - 1; p >= 0; p--)
		if (Union(vzero[p].first, vzero[p].second))
			selZero[p] = 1;
}

void compute(void)
{
	computeConnectedComponents();
	computeInitialOneEdges();
	computeAllOneEdges();
	computeZeroEdges();
}

void writeOutput(void)
{
	freopen("karb.out", "w", stdout);

	for (p = vone.size() - 1; p >= 0; p--)
		if (selOne[p])
			printf("%d %d\n", vone[p].first, vone[p].second);

	for (p = vzero.size() - 1; p >= 0; p--)
		if (selZero[p])
			printf("%d %d\n", vzero[p].first, vzero[p].second);
}

int main()
{
	readInput();
	compute();
	writeOutput();

	return 0;
}
