// O(N^2 * log(N))

#include <stdio.h>
#include <string.h>
#include <vector>
#include <time.h>

using namespace std;

#define NMAX 1024
#define ONE '1'
#define ZERO '0'

#define vmin(a,b) ((a) <= (b) ? (a) : (b))
#define vmax(a,b) ((a) >= (b) ? (a) : (b))

char A[NMAX][NMAX], B[NMAX][NMAX];
short int diagLeft[NMAX][NMAX], diagRight[NMAX][NMAX];
short int val[NMAX][NMAX << 1];
short int left[NMAX << 1], right[NMAX << 1], lson[NMAX << 1], rson[NMAX << 1];
vector<pair<short int, short int> > toRemoveAfter[NMAX];
int i, j, k, a, b, hmax, N, cnt, numPyramids;

void readInput(void)
{
	freopen("piramid.in", "r", stdin);
	scanf("%d", &N);
	for (i = 1; i <= N; i++)
		scanf("%s", &A[i][1]);
}

void update(int col, int node, int value)
{
	while (node >= 1)
	{
		val[col][node] += value;
		node = node >> 1;
	}
}

short int query(int col, int node, int iLeft, int iRight)
{
	short int vLeft, vRight;

	if (left[node] >= iLeft && right[node] <= iRight)
		return val[col][node];
	else
	{
		if (right[lson[node]] >= iLeft)
			vLeft = query(col, lson[node], iLeft, vmin(right[lson[node]], iRight));
		else
			vLeft = 0;

		if (left[rson[node]] <= iRight)
			vRight = query(col, rson[node], vmax(iLeft, left[rson[node]]), iRight);
		else
			vRight = 0;

		return vLeft + vRight;
	}
}

void computeForRotation(void)
{
	// precomputations: diagLeft
	for (i = 1; i <= N; i++)
	{
		k = i; j = 1;
		cnt = 0;

		while (k >= 1 && j <= N)
		{
			if (A[k][j] == ONE)
				cnt++;
			else
				cnt = 0;

			diagLeft[k][j] = cnt;
			k--;
			j++;
		}
	}

	for (i = 2; i <= N; i++)
	{
		k = N; j = i;
		cnt = 0;

		while (k >= 1 && j <= N)
		{
			if (A[k][j] == ONE)
				cnt++;
			else
				cnt = 0;

			diagLeft[k][j] = cnt;
			k--;
			j++;
		}
	}

	// precomputations: diagRight
	for (i = 1; i <= N; i++)
	{
		k = i; j = N;
		cnt = 0;

		while (k >= 1 && j >= 1)
		{
			if (A[k][j] == ONE)
				cnt++;
			else
				cnt = 0;

			diagRight[k][j] = cnt;
			k--;
			j--;
		}
	}

	for (i = 1; i <= N - 1; i++)
	{
		k = N; j = i;
		cnt = 0;

		while (k >= 1 && j >= 1)
		{
			if (A[k][j] == ONE)
				cnt++;
			else
				cnt = 0;

			diagRight[k][j] = cnt;
			k--;
			j--;
		}
	}

	// initialize the segment tree for each column
	memset(val, 0, sizeof(val));

	// clean-up the toRemoveAfter vectors
	for (i = 1; i < NMAX; i++)
		toRemoveAfter[i].clear();

	// actual computations
	for (i = 1; i <= N; i++)
	{
		cnt = 0;
		for (j = 1; j <= N + 1; j++)
			if (A[i][j] == ONE)
				cnt++;
			else
			if (cnt > 0)
			{
				// the interval of columns is [j-cnt,j-1]
				a = j - cnt;
				b = j - 1;

				for (k = a + 1; k <= b - 1; k++)
				{
					hmax = vmin(k - a, b - k);

					// query the column k with the interval [i-hmax,i-1]
					numPyramids += query(k, 1, i - hmax, i - 1);
				}

				cnt = 0;
			}

		// clean-up the segment trees
		for (k = toRemoveAfter[i].size() - 1; k >= 0; k--)
			update(toRemoveAfter[i][k].first, toRemoveAfter[i][k].second + NMAX - 1, -1);
		toRemoveAfter[i].clear();

		// add new values
		for (j = 1; j <= N; j++)
			if (A[i][j] == ONE)
			{
				hmax = vmin(diagLeft[i][j], diagRight[i][j]);
				if (hmax > 1)
				{
					toRemoveAfter[i + hmax - 1].push_back(make_pair((short int)j, (short int)i));
					update(j, i + NMAX - 1, +1);
				}
			}
	}
}

void compute(void)
{
	int r, i, j;

	numPyramids = 0;
	
	for (i = 1; i <= N; i++)
		A[i][N + 1] = ZERO;

	for (i = NMAX; i < 2 * NMAX; i++)
		left[i] = right[i] = i - NMAX + 1;

	for (i = NMAX - 1; i >= 1; i--)
	{
		lson[i] = 2 * i;
		rson[i] = 2 * i + 1;
		left[i] = left[lson[i]];
		right[i] = right[rson[i]];
	}

	// no rotation
	computeForRotation();

	for (r = 1; r <= 3; r++)
	{
		// rotate the matrix 90 degrees up and to the left
		for (i = 1; i <= N; i++)
			for (j = 1; j <= N; j++)
				B[N - j + 1][i] = A[i][j];

		for (i = 1; i <= N; i++)
			for (j = 1; j <= N; j++)
				A[i][j] = B[i][j];

		computeForRotation();
	}
}

void writeOutput(void)
{
	freopen("piramid.out", "w", stdout);
	printf("%d\n", numPyramids);
	fprintf(stderr, "%d\n", numPyramids);
}

int main()
{
	int tstart = clock();

	readInput();
	compute();
	writeOutput();
	
	fprintf(stderr, "Duration=%.4lf sec\n", (double) (clock() - tstart) / CLOCKS_PER_SEC);

	return 0;
}
